{% extends "jb_drf_auth/mailing/base.html" %}

{% block preheader %}Restablece tu contraseña en {{ mailing.brand.app_name|default:"la aplicación" }}.{% endblock %}
{% block header_title %}Restablecer contraseña{% endblock %}

{% block content %}
<p style="margin:0 0 16px 0;font-size:16px;line-height:1.6;color:{{ mailing.theme.text_primary|default:'#212B35' }};">Hola {{ user_email }},</p>
<p style="margin:0 0 20px 0;font-size:16px;line-height:1.6;color:{{ mailing.theme.text_secondary|default:'#637381' }};">Haz clic en el siguiente botón para restablecer tu contraseña:</p>
<p style="margin:0 0 24px 0;">
  <a href="{{ reset_url }}" target="_blank" rel="noopener noreferrer" style="display:inline-block;padding:12px 20px;background:{{ mailing.theme.primary|default:'#0071CE' }};color:#FFFFFF;text-decoration:none;border-radius:6px;font-weight:700;">Restablecer contraseña</a>
</p>
<p style="margin:0;font-size:14px;line-height:1.6;color:{{ mailing.theme.text_secondary|default:'#637381' }};">Si no solicitaste este cambio, ignora este mensaje.</p>
{% endblock %}
